package cn.rankin.resourceservice.service;

import cn.rankin.common.utils.entity.api.APICodeManager;
import cn.rankin.common.utils.entity.api.APIResult;
import cn.rankin.common.utils.entity.page.Page;
import cn.rankin.common.utils.entity.dto.ResourceSearchDTO;
import cn.rankin.resourceservice.dto.ResourceRemote;
import cn.rankin.resourceservice.entity.Resource;
import cn.rankin.resourceservice.proxy.RemoteResourceProxy;
import cn.rankin.resourceservice.repository.ResourceRepository;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Slf4j
@Service
public class ResourceService {

    @Autowired
    private ResourceRepository resourceRepository;

    @Autowired
    private RemoteResourceProxy resourceProxy;

    @Value("${resource.url.domain}")
    private String domain;

    public boolean exists(String code) {
        Long count = resourceRepository.countByCode(code);
        return count > 0L;
    }

    public Resource save(Resource resource) {
        return resourceRepository.save(resource);
    }

    public APIResult<Page<Resource>> search(String code, String name, String sort, Integer pageNo, Integer pageSize) {
        Long count = resourceRepository.count(code, name);
        Page<Resource> page = new Page(count, pageNo, pageSize);

        List<Resource> resourceList = resourceRepository.search(code, name, sort, page.getStart(), pageSize);
        resourceList.forEach(r -> {
            initResourceUrl(r);
        });

        page.setList(resourceList);

        return APIResult.ok(page);
    }

    public void testAPI() {
        Long timeBegin = System.currentTimeMillis();
        resourceProxy.findPageTest(new HashMap<String, Object>() {
            {
                this.put("title", "1");
                this.put("type", 0);
            }
        });
        log.info("not load: {}", System.currentTimeMillis() - timeBegin);
    }

    public APIResult<Page<Resource>> findRemotePage(ResourceSearchDTO searchDTO) {
        Long timeBegin = System.currentTimeMillis();
        APIResult<Page<ResourceRemote>> result = resourceProxy.findPage(new HashMap<String, Object>(){
            {
                this.put("title", searchDTO.getName());
                this.put("pageNum", searchDTO.getPageNo());
                this.put("pageSize", searchDTO.getPageSize());
                this.put("type", searchDTO.getType());
            }
        });
        log.info(JSON.toJSONString(System.currentTimeMillis() - timeBegin));

        if (!result.getSuccess()) {
            log.error("Remote Server Error: code={}, message={}", result.getCode(), result.getMessage());
            APIResult errResult = APIResult.error(APICodeManager.REMOTE_SERVER_ERROR);
            result.setMessage(result.getMessage());
            return errResult;
        }

        Page<ResourceRemote> remotePage = result.getData();
        Page<Resource> page = new Page<>(remotePage.getTotalSize(), remotePage.getPageNo(), remotePage.getPageSize());

        List<Resource> resourceList = new ArrayList<>();
        for (ResourceRemote resourceRemote : remotePage.getList()) {
            resourceList.addAll(resourceRemote.toNativeResource());
        }

        page.setList(resourceList);

        return APIResult.ok(page);
    }

    public APIResult<Page<Resource>> findRemoteByCode(String code) {
        APIResult<ResourceRemote> result = resourceProxy.findByCode(code);
        if (!result.getSuccess()) {
            log.error("Remote Server Error: code={}, message={}", result.getCode(), result.getMessage());
            APIResult errResult = APIResult.error(APICodeManager.REMOTE_SERVER_ERROR);
            result.setMessage(result.getMessage());
            return errResult;
        }

        List<Resource> resourceList = result.getData().toNativeResource();
        Page<Resource> page = new Page((long) resourceList.size());
        page.setList(resourceList);

        return APIResult.ok(page);
    }

    public void initResourceUrl(Resource resource) {
        if (resource == null || domain == null) {
            log.error("domain is null");
            return;
        }else if (resource.getPath() == null) {
            return;
        }

        String path = resource.getPath();
        if (path.startsWith("/")) {
            resource.setUrl(domain + path);
        }else {
            resource.setUrl(domain + '/' + path);
        }
    }
}
